<!-- #include file="../aspctl/sqlite-lib.asp" -->
<!-- #include file="userapi-utils.asp" -->
<!-- #include file="userapi-user.asp" -->
<!-- #include file="userapi-binder.asp" -->
<!-- #include file="userapi-email.asp" -->
<%
    ' USERAPI.ASP
    ' Basic infrastructure for WEB application with user management and multi-language
    '   support.
    '
    ' This file contains the core API for user and rights management/usage and database
    '   access. A single application database is used, but modifications or side usage
    '   of additional database(s) is not excluded.
    '
    ' This API is structured as follows:
    '   Classes are defined in separate files included above. If global instance(s)/singleton of a certain class
    '   is used it is created and initialized in the main file (this one).
    '   Global routines are defined here
    
    ' CONSTANTS FOR DEVELOPMENT TIME CONFIGURATION
    Const cUserDataBase = "/db/arch.db"           ' The database location
    Const cUserAccessAdmin = 100                ' >= admin_level is treated as admin!
    Const cAutoLoginDays = 30                   ' Days to remember the autoligin cookie
    Const cDatabaseNoTempFiles = True           ' Should the use temporary files for transactions
    
    ' Create instances of some frequently used utility objects
    Set StringUtilities = Server.CreateObject("newObjects.utilctls.StringUtilities")
    Set SFMain = Server.CreateObject("newObjects.utilctls.SFMain")
    
    ' Database session initialization callback
    Sub InitDatabaseConnection(db)
        db.Parameters("USER_ID") = CurrentUser.Id
        db.Parameters("GROUP_ID") = CurrentUser.GroupId
        db.Parameters("LANGUAGE") = PageUILanguage
        db.Parameters("USER_LEVEL") = CurrentUser.Level
        db.Parameters("FILTER_SID") = NullConvertTo(vbLong, Session("FILTER_SID"))
        If cDatabaseNoTempFiles Then 
            db.Execute "PRAGMA temp_store=2;"
        End If
    End Sub
    
    Dim Database
    Set Database = New CDatabase
    Database.DatabaseFile = Application("Database")
    Database.SessionInitializer = "InitDatabaseConnection"
    Database.NoMapPath = True
    
    Function GetSID(sid)
        Dim r
        If Database.BeginTransaction Then
            Set r = Database.DB.VExecute("SELECT ID FROM SYS WHERE ID=$ID",1,1,NullConvertTo(vbLong,sid))
            If r.Count > 0 Then
                GetSID = ConvertTo(vbLong, r(1)(1))
            Else
                Set r = Database.DB.VExecute("INSERT INTO SYS (UID) VALUES (?)",1,0,Database.NewUID)
                GetSID = ConvertTo(vbLong, r.Info)
            End If
            If Not Database.CompleteTransaction Then
                GetSID = Null
            End If
        Else
            GetSID = Null
        End If
    End Function
    
    Set CurrentUser = New CUser
    Set CurrentUser.Database = Database
    CurrentUser.LoadFromSession "CURRENT_USER" ' Then ' This gets the preserved user from the session (if any)
    If Not CurrentUser.IsLoggedOn Then
        ' Attempt autologin
        If CurrentUser.AutoLogOn Then
            If Not CurrentUser.SaveToSession("CURRENT_USER") Then
                Response.Write "Failed to save the user data in the session"
            Else
                Database.ReInitialize
            End If
        End If
    End If
    
    Function LogOn(l,p,e,bSave)
        Dim b
        CurrentUser.LogOff
        b = CurrentUser.LogOn(l,p,e)
        LogOn = b
        If b Then
            If Not CurrentUser.SaveToSession("CURRENT_USER") Then
                Response.Write "Failed to save the user data in the session"
            Else
                Database.ReInitialize
            End If
            If bSave Then 
                CurrentUser.SaveAutoLogin CurrentUser.Password
            End If
        End If
    End Function
    Function UpdateCurrentUser
        UpdateCurrentUser = CurrentUser.SaveToSession("CURRENT_USER")
    End Function
    Function LogOff
        CurrentUser.RemoveAutoLogin
        CurrentUser.LogOff
        CurrentUser.SaveToSession "CURRENT_USER"
    End Function
    
    Function AutoLogOn
        Dim b
        AutoLogOn = False
        If CurrentUser.IsLoggedOn Then Exit Function
        If CurrentUser.AutoLogOn Then
            CurrentUser.SaveToSession "CURRENT_USER"
            CurrentUser.SaveAutoLogin CurrentUser.Password
            AutoLogOn = True
        End If
    End Function
    Function RemoveAutoLogOn
        CurrentUser.RemoveAutoLogin
    End Function
    
    Function IsLoggedOn
        IsLoggedOn = CurrentUser.IsLoggedOn
    End Function
    Function IsAdmin
        IsAdmin = CurrentUser.IsAdmin
    End Function
    
    ' Configuration
    
    Class CMainConfiguration
        Private cfg
        Private Sub LoadCfg
            If IsNotObject(cfg) Then
                Set cfg = Database.DB.Execute("SELECT * FROM CONFIGURATION LIMIT 1")(1)
            End If
        End Sub
        Private Sub SaveCfg
            If IsNotObject(cfg) Then Exit Sub
            If Database.BeginTransaction Then
            On Error Resume Next
                Database.DB.CExecute "UPDATE [CONFIGURATION] SET " & CreateSQLAssignList(cfg) & " WHERE ID=$ID", cfg
                Database.CompleteTransaction
            End If
        End Sub
        
        Public Default Property Get Value(n)
            LoadCfg
            Value = cfg(n)
        End Property
        Public Property Let Value(n,v)
            LoadCfg
            cfg(n) = v
            SaveCfg
        End Property
        
        Public Property Get CodeSid(c,msid) ' If msid is Null returns MAIN_SID for code otherwise SID for CODE/MAIN_SID
            Dim r
            Set r = Database.DB.VExecute("SELECT * FROM CONTENT_CONFIGURATION WHERE CODE=$CODE AND ($MSID ISNULL OR MAIN_SID=$MSID) LIMIT 1",1,1,_
                                            NullConvertTo(vbString,c), NullConvertTo(vbLong,msid))
            If r.Count > 0 Then
                If IsNull(NullConvertTo(vbLong,msid)) Then
                    CodeSid = ConvertTo(vbLong, r(1)("MAIN_SID"))
                Else
                    CodeSid = ConvertTo(vbLong, r(1)("OBJECT_SID"))
                End If
            Else
                CodeSid = 0
            End If
        End Property
        Public Property Let CodeSid(c,msid,v)
            Dim m
            m = NullConvertTo(vbLong, msid)
            If Database.BeginTransaction Then
            On Error Resume Next
                If IsNull(m) Then
                    Database.DB.Vexecute "DELETE FROM CONTENT_CONFIGURATION WHERE CODE=$CODE",1,0,NullConvertTo(vbString,c)
                    Database.DB.Vexecute "INSERT INTO CONTENT_CONFIGURATION (CODE,MAIN_SID) VALUES ($CODE,$MSID)",1,0,NullConvertTo(vbString,c),m
                Else
                    Database.DB.Vexecute "DELETE FROM CONTENT_CONFIGURATION WHERE CODE=$CODE AND MAIN_SID=$MSID",1,0,NullConvertTo(vbString,c),m
                    Database.DB.Vexecute "INSERT INTO CONTENT_CONFIGURATION (CODE,MAIN_SID,OBJECT_SID) VALUES ($CODE,$MSID,$OSID)",1,0,NullConvertTo(vbString,c),m,NullConvertTo(vbLong,v)
                End If
                Database.CompleteTransaction
            End If
        End Property
    End Class
    Dim Configuration
    Set Configuration = New CMainConfiguration
    
    
    Function GetDatabaseItemSysInfo(sid, table)
        Dim r
        Set r = Database.DB.VExecute("SELECT SID, CREATED, MODIFIED, R_USER, R_GROUP, R_ALL, DELETED, CHANGED," & _
                                     "  MODIFY_USER_ID, " & _
                                     "  (SELECT LOGIN FROM USER WHERE ID=MODIFY_USER_ID) AS MODIFY_USER_LOGIN, " & _
                                     "  OWNER_USER_ID, " & _
                                     "  (SELECT LOGIN FROM USER WHERE ID=OWNER_USER_ID) AS OWNER_USER_LOGIN, " & _
                                     "  OWNER_GROUP_ID, " & _
                                     "  (SELECT NAME FROM S_GROUP WHERE ID=OWNER_GROUP_ID) AS OWNER_GROUP_NAME " & _
                                     "  FROM [" & Table & "] WHERE SID=$SID AND LANGUAGE=$LANGUAGE",1,1,NullConvertTo(vbLong, sid), PageUILanguage)
        If r.Count > 0 Then
            Set GetDatabaseItemSysInfo = r(1)
        Else
            Set GetDatabaseItemSysInfo = CreateCollection
        End If                                     
    End Function
    Function RightsDisplayText(r,bAll)
        Dim rghts
        rghts = ConvertTo(vbLong, r)
        If rghts >= 4 Then
            RightsDisplayText = TR("write/read")
        ElseIf rghts >= 2 Then
            RightsDisplayText = TR("read")
        ElseIf rghts >= 1 Then
            RightsDisplayText = TR("use")
        Else
            RightsDisplayText = TR("no access")
        End If
    End Function
    
    ' Anonymous IP actions
    Function RequestAnonymousAction(actName,limitNum)
            Dim r, remoteIP, ua, acceptLang
            remoteIP = NullConvertTo(vbString,ASPVARS("REMOTE_ADDR"))
            ua = NullConvertTo(vbString, ASPVARS("HTTP_USER_AGENT"))
            acceptLang = NullConvertTo(vbString, ASPVARS("HTTP_ACCEPT_LANGUAGE"))
            RequestAnonymousAction = False
            
            Set r = Database.DB.VExecute("SELECT *, OleDateDiff('m',PERFORMED,OleSysTime()) AS INTERVAL " & _
                                         " FROM IPACTION WHERE [IP]=$IP AND [ACTION]=$ACTION",1,1, _
                                         remoteIP, NullConvertTo(vbString,actName))
            If r.Count > 0 Then
                If Not ConvertTo(vbBoolean,r(1)("BLOCKED")) Then
                    If ConvertTo(vbLong,r(1)("INTERVAL")) > 1440 Then
                        ' The interval from the last attempt has elapsed - unlock and clear the attempts count
                        Database.DB.VExecute "UPDATE IPACTION SET [ATTEMPTS]=1, [LOCKED]=0, [PERFORMED]=OleSysTime() WHERE " & _
                                             "[IP]=$IP AND [ACTION]=$ACTION", 1, 0, remoteIP, NullConvertTo(vbString,actName)
                        RequestAnonymousAction = True
                    Else
                        If ConvertTo(vbBoolean,r(1)("LOCKED")) Then
                            Exit Function ' Already locked
                        Else
                            If ConvertTo(vbLong, r(1)("ATTEMPTS")) >= limitNum Then
                                ' Lock the IP
                                Database.DB.VExecute "UPDATE IPACTION SET [LOCKED]=-1, [PERFORMED]=OleSysTime() WHERE " & _
                                                     "[IP]=$IP AND [ACTION]=$ACTION", 1, 0, remoteIP, NullConvertTo(vbString,actName)
                                Exit Function
                            Else
                                ' Increase the attempts count
                                Database.DB.VExecute "UPDATE IPACTION SET [ATTEMPTS]=[ATTEMPTS]+1, [PERFORMED]=OleSysTime() WHERE " & _
                                                     "[IP]=$IP AND [ACTION]=$ACTION", 1, 0, remoteIP, NullConvertTo(vbString,actName)
                                RequestAnonymousAction = True
                            End If
                        End If
                    End If
                End If
            Else
                ' Need to insert an initial entry
                Database.DB.Vexecute "INSERT INTO IPACTION ([IP],[ACTION],[PERFORMED],[ATTEMPTS],[UA],[HTTP_ACCEPT_LANGUAGE]) " & _
                                     "VALUES ($IP,$ACTION,OleSysTime(),1,$UA,$HTTP_ACCEPT_LANGUAGE)",1,0, _
                                     remoteIP, NullConvertTo(vbString,actName), ua, acceptLang
                RequestAnonymousAction = True ' The first time is always free
            End If                                         
        End Function
        Function LockAnonymousAction(ip,actName,bLock)
            If bLock Then
                Database.DB.VExecute "UPDATE IPACTION SET [LOCKED]=-1, [PERFORMED]=OleSysTime() WHERE " & _
                                     "[IP]=$IP AND [ACTION]=$ACTION", 1, 0, NullConvertTo(vbString,ip), NullConvertTo(vbString,actName)
            Else
                Database.DB.VExecute "UPDATE IPACTION SET [ATTEMPTS]=0, [LOCKED]=0, [PERFORMED]=OleSysTime() WHERE " & _
                                     "[IP]=$IP AND [ACTION]=$ACTION", 1, 0, NullConvertTo(vbString,ip), NullConvertTo(vbString,actName)
            End If
        End Function
        Function BlockAnonymousAction(ip,actName,bLock)
            If bLock Then
                Database.DB.VExecute "UPDATE IPACTION SET [BLOCKED]=-1 WHERE " & _
                                     "[IP]=$IP AND [ACTION]=$ACTION", 1, 0, NullConvertTo(vbString,ip), NullConvertTo(vbString,actName)
            Else
                Database.DB.VExecute "UPDATE IPACTION SET [BLOCKED]=0, [ATTEMPTS]=0, [LOCKED]=0, [PERFORMED]=OleSysTime() WHERE " & _
                                     "[IP]=$IP AND [ACTION]=$ACTION", 1, 0, NullConvertTo(vbString,ip), NullConvertTo(vbString,actName)
            End If
        End Function
    

%>