<%
' Supply skinning subs for the WEB and User controls
' The subs must follow this prototype:
' Sub Skin_<ClassType>[_<SkinId>](ctl)
' Where:
'   ClassType is what the ClassType property of the control's class (or CValidator) returns (usually this is the control's class name)
'   SkinID part is optional if present this sub is executed only for controls with this SkinId after the routine for the class type. Thus the
'   effect of the routines for a specific SkinId builds over the effect of the general skin for all the controls of that class.
'   All the skin routines receive one parameter named ctl in the examples below. It is the control being skinned - apply the desired 
'   properties over it. These routines are execute when the ApplySkin global routine is called (usually in the beginning of the master page, just
'   before the processing phase begins). Keeping this in mind you can implement quite elaborate skinning that employs more than just static
'   property assignments. For example you can even fill a CList with items from the application's database in its skinning routine.
'   If used well the skin routines can elliminate almost entirely the design and behaviour adjustment related code from the pages and user controls
'   thus making them more readable and easy to manage. Furthermore the skins then can be used as a second level application configuration where
'   the developer can apply quick wholesale changes to the visual representation and behaviour of certain application elements.
'   For example creating skins for the validators will enable you to not just create and apply them with 2 lines of code in the main application code,
'   but also make changes to the messages they display and the conditions they check from a single location - this file.
'
' The examples below are extract from a small real project and illustrate some typical skinning needs. They work with the example style sheet (styles.css)
' included in the template. It is recommended that you clear them all and start anew.


' Text controls
Sub Skin_CText(ctl)
    ctl.CssClass = "InputText"
    If IsEmpty(ctl.Style) Then ctl.Style = "width: 120pt;"
End Sub

Sub Skin_CText_Short(ctl)
    ctl.Style = "width: 100px;"
End Sub

Sub Skin_CText_Normal(ctl)
    ctl.CssClass = "InputText"
    ctl.Style = "width: 120pt;"
End Sub

Sub Skin_CText_Long(ctl)
    ctl.Style = "width: 240pt;"
End Sub

Sub Skin_CText_Longest(ctl)
    ctl.Style = "width: 400pt;"
End Sub

Sub Skin_CText_Filter(ctl)
    ctl.Style = "width: 80pt;"
    ctl.CssClass = "InpCaption"
End Sub
Sub Skin_CText_FilterFull(ctl)
    ctl.Style = "width: 100%;"
    ctl.CssClass = "InpCaption"
End Sub

' Multilang UC-s
Sub Skin_UCMultiLangText_Gray(ctl)
    ctl.CssClass = "UCMulitLang"
End Sub
Sub Skin_UCMultiLangTextArea_Gray(ctl)
    ctl.CssClass = "UCMulitLang"
End Sub
Sub Skin_CTextLang_Gray(ctl)
    ctl.CssClass = "MulitLangField"
End Sub
Sub Skin_CTextAreaLang(ctl)
    ctl.CssClass = "InputTextArea"
End Sub
Sub Skin_CTextAreaLang_Gray(ctl)
    ctl.CssClass = "MulitLangField"
End Sub
Sub Skin_UCLanguage_Gray(ctl)
    ctl.CssClass = "FieldLangSwitch"
End Sub

' CTextArea
Sub Skin_CTextArea(ctl)
    ctl.CssClass = "InputTextArea"
    ctl.Style = "width: 100%; height: 100pt;"
End Sub



' CPass
Sub Skin_CPass(ctl)
    ctl.CssClass = "InputText"
    ctl.Style = "width: 100px;"
End Sub

' Html controls
Sub Skin_CHtmlBox(ctl)
    ctl.Style = "width: 400pt; height: 100pt;"
    ctl.InnerCss = "BODY {border: 1px inset; margin: 1pt; font-family: Arial; font-size: 10pt; }"
End Sub

Sub Skin_CButton(ctl)
    If IsEmpty(ctl.CssClass) Then ctl.CssClass = "InputButton"
    ctl.Style = "width: 140px;text-align: center;"
End Sub
Sub Skin_CButton_Blue(ctl)
    ctl.CssClass = "InputButtonBlue"
End Sub
Sub Skin_CButton_Green(ctl)
    ctl.CssClass = "InputButtonGreen"
End Sub
Sub Skin_CButton_Yellow(ctl)
    ctl.CssClass = "InputButtonYellow"
End Sub
Sub Skin_CButton_Red(ctl)
    ctl.CssClass = "InputButtonRed"
End Sub
Sub Skin_CButton_Orange(ctl)
    ctl.CssClass = "InputButtonOrange"
End Sub

' Image buttons
Sub Skin_CImageButton(ctl)
    If IsEmpty(ctl.Src) Then ctl.Src = SysImage("button")
    If IsEmpty(ctl.CssClass) Then ctl.CssClass = "imageButton"
End Sub
Sub Skin_CImageButton_ImageThumb(ctl)
    ctl.Src = SysImage("pixel")
    ctl.CssClass = "ImageThumb"
    ctl.NoLabel = True
End Sub

Sub Skin_CImageButton_Delete(ctl)
    ctl.Caption = "Delete"
    ctl.Src = SysImage("delete")
    ctl.ConfirmationText = TR("Do you really want to delete this entry?")
End Sub

Sub Skin_CImageButton_PropSheetApply(ctl)
    ctl.Caption = TR("Apply")
    ctl.Src = SysImage("ok")
End Sub
Sub Skin_CImageButton_PropSheetEdit(ctl)
    ctl.Caption = TR("Change")
    ctl.Src = SysImage("edit")
End Sub

' CLists

Sub Skin_CList(ctl)
    ctl.CssClass = "InpClass"
End Sub

' CDateBox
Sub Skin_CDateBox(ctl)
    ctl.CssClass = "InpClass"
    ctl.Style = "width: 80pt;"
    ctl.Format = PageUIDateFormat
End Sub

' Button pager
Sub Skin_ButtonPager(ctl)
    ctl.CssClass = "PagerButton"
    ctl.CurrentCssClass = "PagerButtonCurrent"
    ctl.NoDisable = True
    If IsEmpty(PageSize) Then ctl.PageSize = 10 Else ctl.PageSize = PageSize
    ctl.ShowPages = 4
End Sub

Sub Skin_OrdererBtn(ctl)
    ctl.ShowLabel = True
End Sub

' Validators
Sub Skin_CValidator(ctl)
    ctl.Image = SysImage("err")
    ctl.ShowAllIndicators = True
    If IsEmpty(ctl.ErrorCssClass) Then ctl.ErrorCssClass = "ErrorInpClass"
    If Len(ctl.Text) = 0 Then ctl.Text = TR("Invalid data.") ' This is the default universal text if nothing specific is set.
End Sub

Sub Skin_CValidator_NomenclatureCode(ctl)
    ctl.Message = TR("Nomenclature code must consist of alphanumeric (latin) characters and can be up to 32 symbols long.")
    ctl.Text = TR("The field is invalid")
    ctl.AddProcedure "ValidateRequired" ' Capability to check for required value
    ctl.AddProcedure "ValidateLength"   ' Capability to check the entry length
    ctl.AddProcedure "ValidateRegExp"   ' Capability to check against a regular expression
    ' Parametrsization
    ctl.Required = False
    ctl.MinLength = 1
    ctl.MaxLength = 32
    ctl.RegExpression = "^[a-zA-Z0-9\_\.\-]*$"
End Sub
Sub Skin_CValidator_NomenclatureName(ctl)
    ctl.Message = TR("Nomenclature can be up to 256 symbols long and must be filled in all supported languages.")
    ctl.Text = TR("The field is invalid")
    ctl.AddProcedure "ValidateRequired" ' Capability to check for required value
    ctl.AddProcedure "ValidateLength"   ' Capability to check the entry length
    ctl.AddProcedure "ValidateRegExp"   ' Capability to check against a regular expression
    ' Parametrsization
    ctl.AllLanguages = True
    ctl.Required = True
    ctl.MinLength = 1
    ctl.MaxLength = 256
    ctl.RegExpression = Empty
End Sub
Sub Skin_CValidator_SearchField(ctl)
    ctl.Text = TR("The field is invalid")
    ctl.AddProcedure "ValidateRequired" ' Capability to check for required value
    ctl.AddProcedure "ValidateLength"   ' Capability to check the entry length
    ctl.AddProcedure "ValidateRegExp"   ' Capability to check against a regular expression
    ' Parametrsization
    ctl.Required = True
    ctl.MinLength = 1
    ctl.MaxLength = 128
    ctl.RegExpression = Empty
End Sub
Sub Skin_CValidator_TextArea(ctl)
    ctl.Message = TR("Up to 4096 characters can be specified in a text field")
    ctl.Text = TR("The field is invalid")
    ctl.AddProcedure "ValidateRequired" ' Capability to check for required value
    ctl.AddProcedure "ValidateLength"   ' Capability to check the entry length
    ctl.AddProcedure "ValidateRegExp"   ' Capability to check against a regular expression
    ' Parametrsization
    ctl.MinLength = 1
    ctl.MaxLength = 4096
    ctl.RegExpression = Empty
End Sub

Sub Skin_CValidator_LocationType(ctl)
    ctl.Message = TR("Type is required.")
    ctl.Text = TR("The field is invalid")
    ctl.AddProcedure "ValidateRequired" ' Capability to check for required value
    ' Parametrsization
    ctl.Required = True
End Sub


Sub Skin_CValidator_Description(ctl) ' Standard description
    ' Parametrsization
    If IsEmpty(ctl.Required) Then ctl.Required = True
    ctl.MinLength = 1
    If ConvertTo(vbLong,ctl.MaxLength) = 0 Then ctl.MaxLength = 2048
    ' ctl.RegExpression = "^[a-zA-Z][a-zA-Z0-9\_\-]*$" ' Set one if reg exp should be really checked
    If ctl.Required Then ctl.Message = TR("The description is required.")
    ctl.Message = ctl.Message & TR("Maximum allowed length (in characters) is") & " " & ctl.MaxLength
    ctl.Text = TR("The field is invalid")
    ctl.AddProcedure "ValidateRequired" ' Capability to check for required value
    ctl.AddProcedure "ValidateLength"   ' Capability to check the entry length
    ctl.AddProcedure "ValidateRegExp"   ' Capability to check against a regular expression
End Sub

' Full configuration of a validator in the skin. In the pages/controls you need only to create it, tell it which ontrol to validate and
'   set its SkinId
Sub Skin_CValidator_MyTextField(ctl)
    ctl.Message = TR("The field is invalid. It must not be empty, should be shorter than X symbols and must contain ...")
    ctl.Text = TR("The field is invalid")
    ctl.AddProcedure "ValidateRequired" ' Capability to check for required value
    ctl.AddProcedure "ValidateLength"   ' Capability to check the entry length
    ctl.AddProcedure "ValidateRegExp"   ' Capability to check against a regular expression
    ' Parametrsization
    ctl.Required = True
    ctl.MinLength = 4
    ctl.MaxLength = 16
    ctl.RegExpression = "^[a-zA-Z][a-zA-Z0-9\_\-]*$"
End Sub

' Some skins for more specific elements grouped by purpose
Sub Skin_CImageButton_PanelReload(ctl)
    ctl.Src = VirtPath("/usercontrols/panelcontrols-resources/black/panel-reload.gif")
    ctl.CssClass = "PanelImageButton"
    ctl.NoLabel = True
    ctl.Caption = TR("Refresh list")
End Sub
Sub Skin_CImageButton_PanelEdit(ctl)
    ctl.Src = VirtPath("/usercontrols/panelcontrols-resources/black/panel-edit.gif")
    ctl.CssClass = "PanelImageButton"
    ctl.NoLabel = True
    ctl.Caption = TR("Edit selected")
End Sub
Sub Skin_CImageButton_PanelNew(ctl)
    ctl.Src = VirtPath("/usercontrols/panelcontrols-resources/black/panel-new.gif")
    ctl.CssClass = "PanelImageButton"
    ctl.NoLabel = True
    ctl.Caption = TR("Create new item")
End Sub
Sub Skin_CImageButton_PanelOpen(ctl)
    ctl.Src = VirtPath("/usercontrols/panelcontrols-resources/black/panel-open.gif")
    ctl.CssClass = "PanelImageButton"
    ctl.NoLabel = True
    ctl.Caption = TR("Open and select")
End Sub
Sub Skin_CImageButton_PanelClose(ctl)
    ctl.Src = VirtPath("/usercontrols/panelcontrols-resources/black/panel-close.gif")
    ctl.CssClass = "PanelImageButton"
    ctl.NoLabel = True
    ctl.Caption = TR("Close")
End Sub
Sub Skin_CImageButton_PanelClear(ctl)
    ctl.Src = VirtPath("/usercontrols/panelcontrols-resources/black/panel-clear.gif")
    ctl.CssClass = "PanelImageButton"
    ctl.NoLabel = True
    ctl.Caption = TR("Clear selection")
End Sub

Sub Skin_CList_PanelSelect(ctl)
    ctl.CssClass = "PanelSelect"
End Sub

' Userlist
Sub Skin_CImageButton_GridReload(ctl)
    ctl.Src = VirtPath("/usercontrols/panelcontrols-resources/black/panel-reload.gif")
    ctl.CssClass = "PanelImageButton"
    ctl.NoLabel = True
    ctl.Caption = TR("Refresh list")
End Sub
Sub Skin_CImageButton_UserSelect(ctl)
    ctl.Src = VirtPath("/img/user-on.GIF")
    ctl.CssClass = "PanelImageButton"
    ctl.NoLabel = True
    ctl.Caption = TR("Refresh list")
End Sub
Sub Skin_OrdererBtn_GridOrderer(ctl)
    ctl.ShowLabel = True
End Sub

' Card validators
Sub Skin_CValidator_CardObject(ctl)
    ctl.Message = TR("The object number is invalid or missing")
    ctl.Text = TR("The field is invalid")
    ctl.AddProcedure "ValidateRequired" ' Capability to check for required value
    ctl.AddProcedure "ValidateLength"   ' Capability to check the entry length
    ctl.AddProcedure "ValidateRegExp"   ' Capability to check against a regular expression
    ' Parametrsization
    ctl.Required = True
    ctl.MinLength = 1
    ctl.MaxLength = 16
    ctl.RegExpression = "^[0-9\-]*$"
End Sub

Sub Skin_CValidator_MapScale(ctl)
    ctl.Message = TR("The scale is invalid")
    ctl.Text = TR("The field is invalid")
    ctl.AddProcedure "ValidateRequired" ' Capability to check for required value
    ctl.AddProcedure "ValidateLength"   ' Capability to check the entry length
    ctl.AddProcedure "ValidateRegExp"   ' Capability to check against a regular expression
    ctl.AddProcedure "ValidateType"     ' Capability to check type
    ctl.AddProcedure "ValidateRange"    ' Capability to check range of the value
    ' Parametrsization
    ctl.RegExpression = "^[0-9]*$"
End Sub
Sub Skin_CValidator_Elevation(ctl)
    ctl.Message = TR("The elevation is invalid")
    ctl.Text = TR("The field is invalid")
    ctl.AddProcedure "ValidateRequired" ' Capability to check for required value
    ctl.AddProcedure "ValidateLength"   ' Capability to check the entry length
    ctl.AddProcedure "ValidateRegExp"   ' Capability to check against a regular expression
    ctl.AddProcedure "ValidateType"     ' Capability to check type
    ctl.AddProcedure "ValidateRange"    ' Capability to check range of the value
    ' Parametrsization
    ctl.MinLength = 1
    ctl.MaxLength = 5
    ctl.MinValue = -20000
    ctl.MaxValue = 20000
    ctl.ValType = vbLong
    ctl.RegExpression = "^[0-9\-][0-9]*$"
End Sub
Sub Skin_CValidator_GeoRange(ctl)
    ctl.Message = TR("The range is invalid")
    ctl.Text = TR("The field is invalid")
    ctl.AddProcedure "ValidateRequired" ' Capability to check for required value
    ctl.AddProcedure "ValidateLength"   ' Capability to check the entry length
    ctl.AddProcedure "ValidateRegExp"   ' Capability to check against a regular expression
    ctl.AddProcedure "ValidateType"     ' Capability to check type
    ctl.AddProcedure "ValidateRange"    ' Capability to check range of the value
    ' Parametrsization
    ctl.MinLength = 1
    ctl.MaxLength = 8
    ctl.MinValue = 0
    ctl.MaxValue = 1
    ctl.ValType = vbDouble
    ctl.RegExpression = "^[0-9\.]*$"
End Sub

Sub Skin_CValidator_Area(ctl)
    ctl.Message = TR("The object area is invalid")
    ctl.Text = TR("The field is invalid")
    ctl.AddProcedure "ValidateRequired" ' Capability to check for required value
    ctl.AddProcedure "ValidateLength"   ' Capability to check the entry length
    ctl.AddProcedure "ValidateRegExp"   ' Capability to check against a regular expression
    ctl.AddProcedure "ValidateType"     ' Capability to check type
    ctl.AddProcedure "ValidateRange"    ' Capability to check range of the value
    ' Parametrsization
    ctl.MinLength = 1
    ctl.MaxLength = 6
    ctl.MinValue = 0
    ctl.MaxValue = 1000000
    ctl.ValType = vbDouble
    ctl.RegExpression = "^[0-9\-\.][0-9\.]*$"
End Sub

Sub Skin_CValidator_AnyDate(ctl)
    If IsEmpty(ctl.Message) Then ctl.Message = TR("Invalid date")
    ctl.Text = TR("The field is invalid")
    ctl.AddProcedure "ValidateRequired" ' Capability to check for required value
    ctl.AddProcedure "ValidateDateBox"   ' Capability to check the entry length
    ctl.AddProcedure "ValidateRegExp"   ' Capability to check against a regular expression
    ctl.RegExpression = "^" & GetDateTimeFormatRegExp(PageUIDateFormat) & "$"
    ' Parametrsization
    ' ctl.MinValue = DateSerial(1900,1,1)
    ' ctl.MaxValue = Now
End Sub

Sub Skin_CValidator_CardTextArea(ctl)
    ctl.Message = TR("Up to 4096 characters can be specified in a text field")
    ctl.Text = TR("The field is invalid")
    ctl.AddProcedure "ValidateRequired" ' Capability to check for required value
    ctl.AddProcedure "ValidateLength"   ' Capability to check the entry length
    ctl.AddProcedure "ValidateRegExp"   ' Capability to check against a regular expression
    ' Parametrsization
    ctl.MinLength = 1
    ctl.MaxLength = 4096
    ctl.RegExpression = Empty
End Sub

Sub Skin_CValidator_CardTextLine(ctl)
    ctl.Message = TR("Up to 256 characters can be specified in a single line text field")
    ctl.Text = TR("The field is invalid")
    ctl.AddProcedure "ValidateRequired" ' Capability to check for required value
    ctl.AddProcedure "ValidateLength"   ' Capability to check the entry length
    ctl.AddProcedure "ValidateRegExp"   ' Capability to check against a regular expression
    ' Parametrsization
    ctl.MinLength = 1
    ctl.MaxLength = 256
    ctl.RegExpression = Empty
End Sub

Sub Skin_CValidator_NomComment(ctl)
    ctl.Message = TR("Up to 256 characters can be specified in a single line text field")
    ctl.Text = TR("The field is invalid")
    ctl.AddProcedure "ValidateRequired" ' Capability to check for required value
    ctl.AddProcedure "ValidateLength"   ' Capability to check the entry length
    ctl.AddProcedure "ValidateRegExp"   ' Capability to check against a regular expression
    ' Parametrsization
    ctl.MinLength = 1
    ctl.MaxLength = 256
    ctl.RegExpression = Empty
End Sub

' User controls
Sub Skin_UCVerticalTabMenu_Entry(ctl)
    ctl.CssClass = "vertmenu"
    ctl.ActiveCssClass = "vertmenuActive"
    ctl.ActiveCssClass = "vertmenuActive"
    ctl.AddItem VirtPath("/0-cardhome.asp"),         TR("Card attributes"), Empty, Empty
    ctl.AddItem VirtPath("/1-location.asp"),         TR("Location"), Empty, Empty
    ctl.AddItem VirtPath("/2-localization.asp"),     TR("Localization"), Empty, Empty
    ctl.AddItem VirtPath("/2a-plan.asp"),            TR("Plan"), Empty, Empty
    ctl.AddItem VirtPath("/3-geomorph.asp"),         TR("Geomorphology"), Empty, Empty
    ctl.AddItem VirtPath("/4-info.asp"),             TR("Information"), Empty, Empty
    ctl.AddItem VirtPath("/5-specifics.asp"),        TR("Specifics"), Empty, Empty
    ctl.AddItem VirtPath("/6-type.asp"),             TR("Type and chronology"), Empty, Empty
    ctl.AddItem VirtPath("/7-images.asp"),           TR("Gallery"), Empty, Empty
    ' cardMenu.AddItem "/7-history.asp",          "", Empty, Empty ' Deprecated - moved to Protection
    ctl.AddItem VirtPath("/8-protection.asp"),       TR("Preservation and survey"), Empty, Empty
    ctl.AddItem VirtPath("/9-responsibles.asp"),     TR("Status"), Empty, Empty
End Sub

%>