<%
    Const UCImageDlg_Slim = 0
    Const UCImageDlg_Form = 1
    Const UCImageDlg_List = 2

    ' Shows an image and gives its Id as a value
    ' Allows the user to select an image from the inventory or to upload a new one
    Class UCImageDlg
        
        ' State preservation in GET/QueryString - rarely used in user controls. Should be avoided in controls using too much async post backs
        Public PreserveInQueryString
        Public Property Get HttpGetParams
            HttpGetParams = ""
        End Property
        
        ' Standard fields required by the framework
        Public Name
        Public ClientId ' You can use this in the control rendering code to mark the HTML element that encloses the entire control's contents
        Public CssClass
        Public Style
        Public SkinId
        Private Rendered ' Not currently used - will be removed from the template in future versions
        
        ' PARAMETERS
        Public ShowSelected, BackgroundImage
        
        ' CONTROLS
        Private cImage, cList
        Private btnNew, btnEdit, btnBrowse, btnCancel, btnClear
        
        ' INDICATORS
        Public Changed, Closed ' An image has been selected or the same selection has been left after fiddling with the control
        
        ' INTERNAL
        '   TO DO: Declare fields you need internally for the control functionality
        '   Private myVar
        
        Sub Init
            Rendered = False ' No longer required - about to be deprecated - you can safely delete this line
        
            Set cImage = Create_UCImageForm(Me.Name & "_I")
            Set cList = Create_UCImageList(Me.Name & "_L")
            cList.Columns = ImageBrowsers_Columns
            cList.Rows = ImageBrowsers_Rows
            cList.ListRows = ImageBrowsers_ListRows
            
            Set btnNew = Create_CButton(Empty)
            btnNew.Caption = TR("Upload new image")
            btnNew.AsyncPostBack(Me) = True
            Set btnEdit = Create_CButton(Empty)
            btnEdit.Caption = TR("Change")
            btnEdit.SkinId = "Yellow"
            btnEdit.AsyncPostBack(Me) = True
            Set btnBrowse = Create_CButton(Empty)
            btnBrowse.Caption = TR("Browse inventory")
            btnBrowse.SkinId = "Blue"
            btnBrowse.AsyncPostBack(Me) = True
            Set btnCancel = Create_CButton(Empty)
            btnCancel.Caption = TR("Close")
            btnCancel.SkinId = "Orange"
            btnCancel.AsyncPostBack(Me) = True
            Set btnClear = Create_CButton(Empty)
            btnClear.Caption = TR("Clear")
            btnClear.SkinId = "Orange"
            btnClear.AsyncPostBack(Me) = True
            
            ClientScripts.EnableAsyncLibrary
            
            BackgroundImage = "url(img/imagedlg-background.jpg)"
            
            CssClass = "UCImageDlg"
        End Sub
        
        Public Property Let ImageWidth(v)
            cImage.ImageWidth = v
        End Property
        Public Property Get ImageWidth
            ImageWidth = cImage.ImageWidth
        End Property
        
        ' SESSION Preferences
        ' TO DO: If you do not need session preferences you can remove the 3 members below
        '   Call SavePrefs in the Render phase - as late as possible
        '   Call LoadPrefs in ProcessPage or from outside when the control shows for the first time. Note that choosing the right conditions may be tricky
        '       depending on the way the application is designed. Check the documentation for a discussion on how to find the right conditions.
        Public Sub InitSessionPreferences(vars)
            ' TO DO: Set initial values for the session preferences used by this control
            '   example:
            '   vars("myparam") = "hello"
        End Sub
        Public Sub SavePrefs
            ' TO DO: save the values of the session preferences
            '   example:
            '   SessionPreferences.SetCtlVar Me, "myparam", txtTextBox1.Value
            '   ' A session preference may correspond to a value of a contained control, internal variable or a property
            '   ' In this routine you must fetch it and pass it to the SessionPreferences global singleton class for safekeeping
        End Sub
        Public Sub LoadPrefs
            ' TO DO: Recover the values of the session preferences 
            '   example:
            '   txtTextBox1.Value = ConvertTo(vbString,SessionPreferences.GetCtlVar(Me, "myparam"))
            '   ' When restoring the session preferences you must assign them to their roles inside the control.
            '   ' They may correspond to a value of a contained control (like the example line above), a variable, a property or they
            '   ' can even be only a base over which more than one values are calculated - this is up to you.
        End Sub
        
        ' Load image for editing
        Public Function Load(imgId)
            Load = False
            If cImage.Load(imgId) Then
                Id = imgId
                Mode = UCImageDlg_Form
                cList.Id = 0
                Load = True
            End If
        End Function
        Public Function Browse(imgId)
            Id = imgId
            cList.LoadPrefs
            Mode = UCImageDlg_List
            cList.Id = Id
        End Function
        
        
        Sub ProcessPage
            If btnEdit.Clicked Then
                If Id <> 0 Then
                    Load Id
                End If
            ElseIf btnBrowse.Clicked Then
                Mode = UCImageDlg_List
                cList.LoadPrefs
                cList.Id = Id
            ElseIf btnNew.Clicked Then
                Mode = UCImageDlg_Form
                cImage.Load 0
                cList.Id = 0
            ElseIf cImage.Changed Then
                Id = cImage.Id
                Changed = True
                Closed = True
                Mode = UCImageDlg_Slim
            ElseIf btnCancel.Clicked Then
                Mode = UCImageDlg_Slim
                Changed = True
                Closed = True
            ElseIf btnClear.Clicked Then
                Mode = UCImageDlg_Slim
                cList.Id = 0
                Id = 0
                Changed = True
            ElseIf cList.Selected Then
                Mode = UCImageDlg_Slim
                OvertakePartialRender Me
                Changed = True
                Closed = True
                Id = cList.Id
            End If
        End Sub
        
        ' POST VARIABLES
        Public Property Get Id
           Id = ConvertTo(vbLong,PostVariables.GetCtlVar(Me,"I"))
        End Property
        Public Property Let Id(v)
           PostVariables.SetCtlVar Me, "I", ConvertTo(vbLong,v)
        End Property
        Public Property Get Mode
            Mode = ConvertTo(vbLong,PostVariables.GetCtlVar(Me,"M"))
        End Property
        Public Property Let Mode(v)
            PostVariables.SetCtlVar Me, "M", ConvertTo(vbLong,v)
        End Property
        
        
        ' TO DO: implement or leave empty the Value property
        Public Property Get Value
            Value = Empty
            If Id <> 0 Then Value = Id
        End Property
        Public Property Let Value(v)
            Id = v
        End Property
        
        
        Public Property Get ClassType
            ' TO DO: Change the class name to the unique name of your class. Recommendation place the same name as the declared VBScript class name
            ClassType = "UCImageDlg" 
        End Property
        
        ' TO DO: If you are not going to use PreRender phase you can safely remove this method
        Sub PreRender(param) ' The param is always empty in the current versions of ASP-CTL
            ' PreRender is most often needed by controls that generate a lot of client side javascript code.
            ' This is the place where they can safely assume the processing is finished and any bindings between the client code and server elements
            ' can be determined without risk that they might change later.
        End Sub
        
        Public Sub Render
            ' Renders the client side container for the control
            ' TO DO: Change the code if needed - e.g. use span, table or something else more appropriate if div is not suitable
            %>
            <div <%= StyleAndCssString(CssClass,Style) %> id="<%= ClientId %>">
                <% RenderPartial %>
            </div>
            <%
            ' SavePrefs ' If you are using session preferences this is the best place to save them.
        End Sub
        Public Sub RenderPartial
            If Mode = UCImageDlg_Slim Then
                RenderSlim
            ElseIf Mode = UCImageDlg_Form Then
                If ShowSelected Then
                    RenderSlim
                End IF
                cImage.Render
            ElseIf Mode = UCImageDlg_List Then
                If ShowSelected Then
                    RenderSlim
                End If
                cList.Render
                ' btnCancel.Render
            End If
        End Sub
        Public Sub RenderSlim
            Dim r, rec
            If Id <> 0 Then
                Set r = Database.DB.VExecute("SELECT I.*, T.NAME AS IMAGE_TYPE_CAPTION FROM IMAGE I LEFT OUTER JOIN IMAGE_TYPE T ON I.IMAGE_TYPE_SID=T.SID AND T.LANGUAGE=$LANGUAGE WHERE " & _
                                             " NOT I.DELETED AND I.SID=$SID AND I.LANGUAGE=$LANGUAGE AND " & SQLReadRightsTable("I"),1,1,PageUILanguage,Id)
                If r.Count > 0 Then
                    Set rec = r(1)
                Else
                    Set rec = CreateCollection
                End If
            Else
                Set rec = CreateCollection
            End If
            %>
            <table class="form" style="background-image: <%= BackgroundImage %>;background-position: top right; background-repeat: no-repeat;background-color: #FFFFFF;">
                <tr>
                    <td class="formLabel" style="vertical-align: top;">
                        <img src="<%= VirtPath("/image.asp") & "?image=" & Id & "&th=2" %>" 
                        alt="<%= HTMLEncode(rec("NAME")) %>" class="EntryThumbnail"/>
                    </td>
                    <td class="formField" style="width:100%; vertical-align: top;">
                        <table class="viewPanel">
                            <tr>
                                <td class="buttons" width="100%" colspan="2">
                                    <% 
                                        btnNew.Render
                                        If Id <> 0 Then btnEdit.Render
                                        btnBrowse.Render
                                        If Id <> 0 Then btnClear.Render
                                    %>
                                </td>
                            </tr>
                            <tr>
                                <td class="content" colspan="2">
                                    <% If Id <> 0 Then %>
                                        <strong><%= HTMLEncode(rec("NAME")) %></strong><br/>
                                        <%= rec("WIDTH") & " x " & rec("HEIGHT") %>
                                        <% 
                                            If ShowBBCode Then 
                                                Response.Write "<br/>BBCode: <strong>[image " & rec("ID") & "]</strong>"
                                            End If
                                        %>
                                            
                                    <% Else %>
                                        <%= TR("No image selected") %>
                                    <% End If %>
                                </td>
                            </tr>
                        </table>
                    </td>
                </tr>
            </table>
            <%
        End Sub
    
    End Class
    
    ' Constructor
    Function Create_UCImageDlg(ctlName) 
        Set ctl = New UCImageDlg ' TO DO: Change the control name
        ctl.Name = IfThenElse(Len(ctlName) > 0,ctlName,NewCtlName)
        ctl.ClientId = NewClientId()
        RegisterUserControl ctl
        ctl.Init ' The initialization must happen after registration
        UserControlProcessLast ctl ' Moves this control after any contained user controls it may have created in the processing chain
        Set Create_UCImageDlg = ctl ' TO DO: Change the control name
    End Function

%>