<%
    Class UCImageType
        
        ' User controls should not usually preserve state in the query string.
        Public PreserveInQueryString
        Public Property Get HttpGetParams
            HttpGetParams = ""
        End Property
        
        ' Standard fields required by the framework
        Public Name
        Public ClientId ' You can use this in the control rendering code to mark the HTML element that encloses the entire control's contents
        Private Rendered ' See the documentation or just ignore it
        Public CssClass
        Public Style
        Public SkinId
        
        Public HideReload
        
        ' CONTROLS
        Private lstTypes, btnReload, btnAddNew, btnEdit
        Private txtWidth, txtHeight, txtCode, lstContentType, txtFileExt
        Private vldWidth, vldHeight, vldCode, vldContentType, vldFileExt
        Private vldWidthHeight
        Private txtCaption, vldCaption
        Private btnOk, btnCancel, btnDelete
        
        Private binder
        Private cRights
        
        ' INDICATORS
        Public Changed, Selected, Cancelled
        
        Sub Class_Initialize
            Set cRights = Create_UCObjectRights(Empty)
            cRights.Table = "IMAGE_TYPE"
        
            Me.Name = NewCtlName()
            RegisterUserControl Me
            ClientId = NewClientId()
            Rendered = False
            
            Set lstTypes = Create_CList(Empty)
            lstTypes.AddItem "", ""
            lstTypes.CssClass = "SelectorBar"
            FillNomenclature
            
            Set btnReload = Create_CImageButton(Empty)
            btnReload.Caption = "Change"
            btnReload.SkinId = "PanelReload"
            btnReload.AsyncPostBack(Me) = True
            Set btnAddNew = Create_CImageButton(Empty)
            btnAddNew.SkinId = "PanelNew"
            btnAddNew.AsyncPostBack(Me) = True
            Set btnEdit = Create_CImageButton(Empty)
            btnEdit.SkinId = "PanelEdit"
            btnEdit.AsyncPostBack(Me) = True
            
            Set btnOk = Create_CButton(Empty)
            btnOk.Caption = TR("Ok")
            btnOk.SkinId = "Green"
            btnOk.AsyncPostBack(Me) = True
            Set btnCancel = Create_CButton(Empty)
            btnCancel.Caption = TR("Cancel")
            btnCancel.SkinId = "Orange"
            btnCancel.AsyncPostBack(Me) = True
            Set btnDelete = Create_CButton(Empty)
            btnDelete.Caption = TR("Delete")
            btnDelete.SkinId = "Red"
            btnDelete.ConfirmationText = TR("Do you want to delete this image type?")
            btnDelete.AsyncPostBack(Me) = True
            
            ' Form controls
            Set txtWidth = Create_CText(Empty)
            txtWidth.SkinId = "Short"
            Set vldWidth = Create_NumericValidator(txtWidth,4,False,vbLong,16,2048)
            vldWidth.Group = Me.Name
            vldWidth.Message = TR("Max width must be between 16 and 2048")
            Set txtHeight = Create_CText(Empty)
            txtHeight.SkinId = "Short"
            Set vldHeight = Create_NumericValidator(txtHeight,4,False,vbLong,16,2048)
            vldHeight.Group = Me.Name
            vldHeight.Message = TR("Max height must be between 16 and 2048")
            
            Set vldWidthHeight = Create_CValidatorProc("ValidateRequireSome")
            With vldWidthHeight
                .Required = True
                .MinValue = 1
                .Group = Me.Name
                .AddControlToValidate txtWidth
                .AddControlToValidate txtHeight
                .Message = TR("At least one Max Width/Height must be specified")
            End With
            
            Set txtCode = Create_CText(Empty)
            txtCode.SkinId = "Normal"
            Set vldCode = Create_TextValidator(txtCode,16,False,1,16,"^[A-Za-z][A-Za-z0-9\_]*$")
            vldCode.Group = Me.Name
            vldCode.Message = TR("The application code must be alphanumeric (beginning with an alpha character")
            Set lstContentType = Create_CList(Empty)
            With lstContentType
                .AddItem "", ""
                .AddItem "image/jpeg",  "JPEG Images"
                .AddItem "image/gif",   "GIF Images"
                .AddItem "image/png",   "PNG Images"
            End With
            Set vldContentType = Create_RequiredValidator(lstContentType,Empty,True)
            vldContentType.Group = Me.Name
            vldContentType.Message = TR("The content type is required")
            
            Set txtCaption = Create_UCMultiLangText(Empty)
            txtCaption.SkinId = "Gray"
            txtCaption.Style = "width: 100%;"
            Set vldCaption = Create_TextValidator(txtCaption,MaxCaptionSize,True,0,MaxCaptionSize,Empty)
            vldCaption.Group = Me.Name
            vldCaption.Message = TR("The caption is required. It can be up to 255 characters long.")
            
            Set txtFileExt = Create_CText(Empty)
            txtFileExt.SkinId = "Short"
            Set vldFileExt = Create_TextValidator(txtFileExt,32,False,0,32,"^[a-zA-Z0-9\_\-]*$")
            vldFileExt.Group = Me.Name
            vldFileExt.Message = TR("The file extension may consist of latin letters, numbers and the _ and - characters. It can be up to 32 characters long.")
            
            Set binder = Create_CRecordBinder("LANGUAGE")
            binder.BindControl txtCode, vbString, "CODE", vbString, "Default", "Default"
            binder.BindControl txtCaption, vbString, "NAME", vbString, "Default", "Default"
            binder.BindControl txtWidth, vbString, "WIDTH", vbLong, "Default", "Default"
            binder.BindControl txtHeight, vbString, "HEIGHT", vbLong, "Default", "Default"
            binder.BindControl txtFileExt, vbString, "FILE_EXTENSION", vbString, "Default", "Default"
            binder.BindControl lstContentType, vbString, "CONTENT_TYPE", vbString, "Default", "Default"
            
            Set lstTypes.AsyncPostBack(Me) = btnReload
            
            ClientScripts.EnableAsyncLibrary
            
            CssClass = "UCImageType"
        End Sub
        
        Private Sub FillNomenclature
            lstTypes.Items.Clear
            lstTypes.AddItem "", ""
            lstTypes.AddSQLiteItems Database.DB.VExecute("SELECT SID, NAME FROM [IMAGE_TYPE] WHERE NOT DELETED AND LANGUAGE=$LANGUAGE AND " & _
                                                         SQLReadRights & " ORDER BY NAME",1,0,PageUILanguage), "SID", "NAME"
        End Sub
        
        Public Function Load(sid)
            Dim r
            Load = False
            Set r = binder.ItemForEdit(sid,"IMAGE_TYPE")
            If r Is Nothing Then Exit Function
            binder.ReadFrom r
            Id = sid
            txtCaption.Language = PageUILanguage
            cRights.Id = Id
            EditMode = True
            Load = True
        End Function
        Public Function Save
            Save = False
            If Validate(Me.Name) Then
                If Database.BeginTransaction Then
                On Error Resume Next
                    Database.FailureValue(Me,"Id") = Id
                    Set r = binder.ItemForWrite(Id, "IMAGE_TYPE")
                    
                    If Not r Is Nothing Then
                        binder.WriteTo r
                        Id = binder.WriteItem(r, "IMAGE_TYPE")
                    End If
                    
                    If Database.CompleteTransaction Then
                        Save = True
                    End If
                    On Error Goto 0
                End If
            Else
                Database.InvalidateTransaction
            End If
        End Function
        Public Function Delete
            Delete = False
            If ImageTypeId <> 0 Then
                If Database.BeginTransaction Then
                On Error Resume Next
                Err.Clear
                    Database.FailureValue(Me,"Id") = Id
                    binder.DeleteItem Id, "IMAGE_TYPE"
                    Id = 0
                    If Database.CompleteTransaction Then
                        Delete = True
                    End If
                    On Error Goto 0
                End If
            End If
        End Function
        
        Public Function Complete(bSuccess, Failure, Success)
            Complete = True
            If Not bSuccess Then
                Id = ConvertTo(vbLong, Failure("Id"))
                cRights.Id = Id
            End If
        End Function
        
        Sub ProcessPage
            Dim r, sid
            If IsPostBack Then
                If Not EditMode Then
                    ImageTypeId = ConvertTo(vbLong,lstTypes.SelectedValue)
                End If
                
                If btnEdit.Clicked Then
                    Load Id
                ElseIf btnAddNew.Clicked Then
                    Load 0
                ElseIf EditMode And btnOk.Clicked Then
                    If Save Then
                        FillNomenclature
                        EditMode = False
                        Changed = True
                        cRights.Id = Id
                        Selected = True
                    End If
                ElseIf btnDelete.Clicked Then
                    If Delete Then
                        FillNomenclature
                        EditMode = False
                        Changed = True
                        cRights.Id = Id
                        Selected = True
                    End If
                ElseIf btnCancel.Clicked Then
                    EditMode = False
                    Cancelled = True
                End If
            End If
        End Sub
        
        ' PROPERTIES
        Public Property Let Auto(v)
            If v Then
                Set lstTypes.AsyncPostBack(Me) = btnReload
            Else
                lstTypes.AsyncPostBack(Me) = False
            End If
        End Property
        
        Public Property Get Id
            If Not EditMode Then
                Id = ConvertTo(vbLong,lstTypes.SelectedValue)
            Else
                Id = ImageTypeId
            End If
        End Property
        Public Property Let Id(v)
            lstTypes.SelectedValue = ConvertTo(vbLong, v)
            ImageTypeId = ConvertTo(vbLong, v)
        End Property
        
        ' POST VARIABLES
        Public Property Get ImageTypeId
            ImageTypeId = ConvertTo(vbLong,PostVariables.GetCtlVar(Me,"I"))
        End Property
        Public Property Let ImageTypeId(v)
            PostVariables.SetCtlVar Me, "I", ConvertTo(vbLong, v)
        End Property
        Public Property Get EditMode
            EditMode = ConvertTo(vbBoolean, PostVariables.GetCtlVar(Me,"M"))
        End Property
        Public Property Let EditMode(v)
            b = ConvertTo(vbBoolean, v)
            PostVariables.SetCtlVar Me, "M", b
            If b Then
                
            End If
        End Property
        
        ' Language
        
        
        Public Property Get Value
            Value = Id
            If Id = 0 Then Value = Empty
        End Property
        Public Property Let Value(v)
            Id = v
        End Property
        
        
        Public Property Get ClassType
            ClassType = "UCImageType" 
        End Property
        
        Public Sub Render
            %>
            <div <%= StyleAndCssString(CssClass,Style) %> Id="<%= ClientId %>">
                <% RenderPartial %>
            </div>
            <%
        End Sub
        Public Sub RenderPartial
            If EditMode Then
                RenderForm
            Else
                RenderSelector
            End IF
        End Sub
        
        
        Private Sub RenderSelector
            %>
            <table class="SelectorBar">
                <tr>
                    <td class="leftSide"><img class="pixel" src="<%= SysImage("pixel") %>" alt=""/></td>
                    <td nowrap><% btnAddNew.Render %></td>
                    <% 
                        If Id <> 0 Then 
                            %><td nowrap><% btnEdit.Render %></td><%
                        End If
                    %>
                    <td nowrap><% lstTypes.Render %></td>
                    <% If Not HideReload Then %>
                        <td nowrap><% btnReload.Render %></td>                        
                    <% End If %>
                    <td nowrap width="100%">&nbsp;</td>
                    <td class="rightSide"><img class="pixel" src="<%= SysImage("pixel") %>" alt=""/></td>
                </tr>
            </table>
            <%
        End Sub
        Private Sub RenderForm
            %>
                <table class="form" >
                    <tr>
                        <th class="formCaption" colspan="2"><%= TR("Editing an image type definition") %></th>
                    </tr>
                    <tr>
                        <th nowrap class="formLabel" colspan="2"><% cRights.Render %></th>
                    </tr>
                    <tr>
                        <th nowrap class="formLabel" colspan="2"><%= TR("Caption") %><% vldCaption.Render %></th>
                    </tr>
                    <tr>
                        <td nowrap class="formField" colspan="2"><% txtCaption.Render %></td>
                    </tr>
                    <tr>
                        <th nowrap class="formLabel"><%= TR("Max Width") %></th>
                        <td nowrap class="formField"><% txtWidth.Render : vldWidth.Render : vldWidthHeight.Render %></td>
                    </tr>
                    <tr>
                        <th nowrap class="formLabel"><%= TR("Max Height") %></th>
                        <td nowrap class="formField"><% txtHeight.Render : vldHeight.Render : vldWidthHeight.Render %></td>
                    </tr>
                    <tr>
                        <th nowrap class="formLabel"><%= TR("Code") %></th>
                        <td nowrap class="formField"><% txtCode.Render : vldCode.Render %></td>
                    </tr>
                    <tr>
                        <th nowrap class="formLabel"><%= TR("File extension") %></th>
                        <td class="formField"><% txtFileExt.Render : vldFileExt.Render %><br/>
                        <%= TR("Enter an extension that matches the content type.") %>
                        <%= TR("This is not required, but allows the uploaded image files to be directly accessible if desired.") %>
                        </td>
                    </tr>
                    <tr>
                        <th nowrap class="formLabel"><%= TR("Content type") %></th>
                        <td nowrap class="formField"><% lstContentType.Render : vldContentType.Render %><br/>
                        <%= TR("The images are converted to the specified image format.") %>
                        </td>
                    </tr>
                    <tr>
                        <td nowrap class="formButtons" colspan="2">
                            <% 
                                btnOk.Render
                                btnCancel.Render
                                If Id <> 0 Then btnDelete.Render
                            %>
                        </td>
                    </tr>
                </table>
            <%
        End Sub
    
    End Class

%>