<%
    ' Allows selection of a language - for use by other user controls which allow the user to change the language for which data is entered
    Class UCLanguage
        
        ' User controls should not usually preserve state in the query string.
        Public PreserveInQueryString
        Public Property Get HttpGetParams
            HttpGetParams = ""
        End Property
        
        ' Standard fields required by the framework
        Public Name
        Public ClientId ' You can use this in the control rendering code to mark the HTML element that encloses the entire control's contents
        Private Rendered ' See the documentation or just ignore it
        Public CssClass
        Public Style
        Public SkinId
        
        ' PROPERTIES
        Public Disabled ' Changes are disallowed
        Public SwitchUILanguage
        Public Label
        Public LanguageControl ' (optional) A control supporting the PLanguageControl protocol. 
        
        ' CHILDREN / PARENTS
        Private collChildren, Parent ' Parent language switcher
        
        ' CONTROLS
        Private btnLanguage
        
        Private PreviousLanguage
        
        ' INDICATORS
        Public Changed
        
        Sub Class_Initialize
            Set collChildren = CreateCollection
            
            Me.Name = NewCtlName()
            RegisterUserControl Me
            ClientId = NewClientId()
            Rendered = False
            
            Set btnLanguage = Create_CImageButton(Empty)
            btnLanguage.NoLabel = True
            btnLanguage.CssClass = "LanguageBarIndicator"
            
            Label = TR("Language")
            
            CssClass = "UCLanguage"
        End Sub
        
        Function AddChild(ctl)
            Set collChildren(ctl.Name) = ctl
            Set ctl.Parent = Me
        End Function
        Function RemoveChild(ctl)
            collChildren.Remove ctl.Name
            Set ctl.Parent = Nothing
        End Function
        Function AdviseChildren
            Dim I
            For I = 1 To collChildren.Count
                collChildren(I).Language = Language
                collChildren(I).Changed = Changed
                collChildren(I).AdviseChildren
            Next
        End Function
        Function CancelLanguageChange
            If IsNotObject(Parent) Then
                If Len(OldLanguage) <> 0 Then 
                    Language = OldLanguage
                    Changed = False
                    AdviseChildren
                End If
            Else
                Parent.CancelLanguageChange
            End If
        End Function
        Public Property Get HasParent
            HasParent = Not IsNotObject(Parent)
        End PRoperty
        
        ' Async post back
        Public Property Get AsyncPostBack(ctl)
            AsyncPostBack = btnLanguage.AsyncPostBack(ctl)
        End Property
        Public Property Let AsyncPostBack(ctl,v)
            btnLanguage.AsyncPostBack(ctl) = v
        End Property
        
        
        Sub ProcessPage
            Dim lng
            If btnLanguage.Clicked Then
                lng = ConvertTo(vbString, btnLanguage.Value)
                If lng <> Language Then 
                    Changed = True
                    AdviseChildren
                End If
                OldLanguage = Language
                Language = lng
                If SwitchUILanguage And OldLanguage <> Language Then
                    SetSessionLanguage Language
                End IF
            End If
        End Sub
        
        Sub MasterProcessPage
            If btnLanguage.Clicked Then
                lng = ConvertTo(vbString, btnLanguage.Value)
                If lng <> Language Then 
                    SetSessionLanguage lng
                End If
            End If
        End Sub
        
        ' PROPERTIES
        Public Property Get Language
            Dim lng
            lng = ConvertTo(vbString, PostVariables.GetCtlVar(Me,"L"))
            If Len(lng) <= 0 Then
                Language = PageUILanguage ' To allow safe calls before processing -  because it can be done
            Else
                Language = lng
            End If
        End Property
        Public Property Let Language(v)
            PostVariables.SetCtlVar Me, "L", ConvertTo(vbString, v)
        End Property
        Public Property Get LanguageChanged
            LanguageChanged = Changed
        End Property
        
        
        Public Default Property Get Value
            Value = Language
        End Property
        
        Public Property Get ClassType
            ClassType = "UCLanguage" 
        End Property
        
        Public Sub Render
            Dim r, I
            btnLanguage.NoLabel = True
            Set r = ASPCTLResDB.Execute("SELECT LANGUAGE, NAME || '(' || NAMENATIVE || ')' AS NAME FROM Languages")
            %>
            <table <%= StyleAndCssString(CssClass,Style) %>>
                <tr>
                    <td class="leftSide"><img class="pixel" src="<%= SysImage("pixel") %>" alt=""/></td>
                    <td nowrap style="width: 100%; vertical-align: middle;"><%= Label %></td>
                    <td nowrap class="nonselected"></td>
                    <% 
                        If IsObject(LanguageControl) Then
                            For I = 1 To r.Count
                                ' btnLanguage.Src = VirtPath("/img/" & r(I)("LANGUAGE") & ".gif")
                                btnLanguage.Src = LanguageIcon(r(I)("LANGUAGE"))
                                btnLanguage.Caption = r(I)("NAME")
                                If ConvertTo(vbString, r(I)("LANGUAGE")) = Language Then
                                    %>
                                    <td nowrap class="selected"><% btnLanguage.RenderValue(r(I)("LANGUAGE")) %></td>
                                    <%
                                Else
                                    If Len(LanguageControl.LanguageValue(r(I)("LANGUAGE"))) > 0 Then
                                    %>
                                    <td nowrap class="nonempty" ><% btnLanguage.RenderValue(r(I)("LANGUAGE")) %></td>
                                    <%
                                    Else
                                    %>
                                    <td nowrap class="nonselected" ><% btnLanguage.RenderValue(r(I)("LANGUAGE")) %></td>
                                    <%
                                    End If
                                End If
                            Next
                        Else
                            For I = 1 To r.Count
                                ' btnLanguage.Src = VirtPath("/img/" & r(I)("LANGUAGE") & ".gif")
                                btnLanguage.Src = LanguageIcon(r(I)("LANGUAGE"))
                                btnLanguage.Caption = r(I)("NAME")
                                If ConvertTo(vbString, r(I)("LANGUAGE")) = Language Then
                                    %>
                                    <td nowrap class="selected"><% btnLanguage.RenderValue(r(I)("LANGUAGE")) %></td>
                                    <%
                                Else
                                    %>
                                    <td nowrap class="nonselected" ><% btnLanguage.RenderValue(r(I)("LANGUAGE")) %></td>
                                    <%
                                End If
                            Next
                        End If
                    %>
                    <td class="rightSide"><img class="pixel" src="<%= SysImage("pixel") %>" alt=""/></td>
                </tr>
            </table>
            <%
        End Sub
    
    End Class

%>