<%
    ' Entity editing control
    ' Mode - 0 - slim, 1 - list, 2 - form
    Const UCPerson_Mode_Slim = 0
    Const UCPerson_Mode_List = 1
    Const UCPerson_Mode_Form = 2
    
    Function Create_UCPerson(ctlName) 
        Set ctl = New UCPerson ' TO DO: Change the control name
        ctl.Name = IfThenElse(Len(ctlName) > 0,ctlName,NewCtlName)
        ctl.ClientId = NewClientId()
        RegisterUserControl ctl
        ctl.Init ' The initialization must happen after registration
        UserControlProcessLast ctl ' Moves this control after any contained user controls it may have created in the processing chain
        Set Create_UCPerson = ctl ' TO DO: Change the control name
    End Function
    
    Class UCPerson
        
        ' User controls should not usually preserve state in the query string.
        Public PreserveInQueryString
        Public Property Get HttpGetParams
            HttpGetParams = ""
        End Property
        
        ' Standard fields required by the framework
        Public Name
        Public ClientId ' You can use this in the control rendering code to mark the HTML element that encloses the entire control's contents
        Private Rendered ' See the documentation or just ignore it
        Public CssClass
        Public Style
        Public SkinId
        
        ' PARAMETERS
        Public ReadOnly
        Public Hide
        
        ' FORM FIELDS - name1 is used for name where only one exists
        Private txtName1, txtName2, txtName3, txtTitle
        Private vldName1, vldName2, vldName3, vldTitle
        Private vldNamesPerson
        Private txtComment, vldComment
        Private cImage
        Private cRights
        Private cPersons
        Private binder
        Private btnOk, btnCancel, btnDelete
        Private btnOpen, btnClose, btnEdit, btnAddNew, btnClear
        
        ' INDICATORS
        Public Selected
        
        ' TO DO: Declare fields you need internally for the control functionality
        
        Sub Init
            Set cPersons = Create_UCPersons(Me.Name & "_L")
            Set cImage = Create_UCImageDlg(Me.Name & "_I")
            Set cRights = Create_UCObjectRights(Empty)
            Rendered = False
            cRights.Table = "PERSON"
            
            ' Person Form
            Set txtName1 = Create_UCMultiLangText(Empty)
            txtName1.SkinId = "Gray"
            Set vldName1 = Create_UCValidator(txtName1,Me,"PersonName")
            Set txtName2 = Create_UCMultiLangText(Empty)
            txtName2.SkinId = "Gray"
            Set vldName2 = Create_UCValidator(txtName2,Me,"PersonName")
            Set txtName3 = Create_UCMultiLangText(Empty)
            txtName3.SkinId = "Gray"
            Set vldName3 = Create_UCValidator(txtName3,Me,"PersonName")
            Set txtTitle = Create_UCMultiLangText(Empty)
            txtTitle.SkinId = "Gray"
            Set vldTitle = Create_UCValidator(txtTitle,Me,"PersonTitle")
            
            ' Assures at least two names are specified
            Set vldNamesPerson = Create_CValidatorProc("ValidateRequireSome")
            With vldNamesPerson
                .Required = True
                .MinValue = 2
                .Group = Me.Name
                .AddControlToValidate txtName1
                .AddControlToValidate txtName2
                .AddControlToValidate txtName3
                .AddControlToValidate txtTitle
                .Message = TR("At least two names must be specified")
            End With

            Set txtComment = Create_UCMultiLangTextArea(Empty)
            txtComment.SkinId = "Gray"
            Set vldComment = Create_UCValidator(txtComment, Me, "TextArea")
            
            Set binder = New CRecordBinder
            binder.BindControl txtComment, vbString, "COMMENT", vbString, "Default", "Default"
            binder.BindControl txtName1, vbString, "NAME1", vbString, "Default", "Default"
            binder.BindControl txtName2, vbString, "NAME2", vbString, "Default", "Default"
            binder.BindControl txtName3, vbString, "NAME3", vbString, "Default", "Default"
            binder.BindControl txtTitle, vbString, "TITLE", vbString, "Default", "Default"
            binder.BindControl cImage, vbLong, "IMAGE_SID", vbLong, "Default", "Default"

            ' Buttons
            Set btnEdit = Create_CImageButton(Empty)
            btnEdit.SkinId = "PanelEdit"
            btnEdit.AsyncPostBack(Me) = True
            Set btnAddNew = Create_CImageButton(Empty)
            btnAddNew.SkinId = "PanelNew"
            btnAddNew.AsyncPostBack(Me) = True
            Set btnClear = Create_CImageButton(Empty)
            btnClear.SkinId = "PanelClear"
            btnClear.AsyncPostBack(Me) = True
            
            Set btnOpen = Create_CImageButton(Empty)
            btnOpen.SkinId = "PanelOpen"
            btnOpen.AsyncPostBack(Me) = True
            Set btnClose = Create_CImageButton(Empty)
            btnClose.SkinId = "PanelClose"
            btnClose.AsyncPostBack(Me) = True
            
            Set btnOk = Create_CButton(Empty)
            btnOk.Caption = TR("Ok")
            btnOk.SkinId = "Green"
            btnOk.AsyncPostBack(Me) = True
            Set btnCancel = Create_CButton(Empty)
            btnCancel.Caption = TR("Cancel")
            btnCancel.SkinId = "Orange"
            btnCancel.AsyncPostBack(Me) = True
            Set btnDelete = Create_CButton(Empty)
            btnDelete.Caption = TR("Delete")
            btnDelete.SkinId = "Red"
            btnDelete.AsyncPostBack(Me) = True
            btnDelete.ConfirmationText = TR("Do you really want to remove this item?")
            
            CssClass = "UserControl"
            
        End Sub
        
        
        Public Property Get Users
            Users = cPersons.Users
        End Property
        Public Property Let Users(v)
            cPersons.Users = v
        End Property
        
        Public Function Load(sid)
            Dim r
            Load = False
            If ReadOnly Then Exit Function
            Set r = binder.ItemForEdit(sid,"PERSON")
            If r Is Nothing Then Exit Function
            binder.ReadFrom r
            Id = sid
            txtComment.Language = PageUILanguage
            cRights.Id = Id
            Mode = UCPerson_Mode_Form
            Load = True
        End Function
        Public Function Browse(sid)
            cPersons.Id = sid
            Mode = UCPerson_Mode_List
        End Function
        
        Public Function Save
            Dim r
            Save = False
            If Validate(Me.Name) Then
                If Database.BeginTransaction Then
                On Error Resume Next
                    Database.FailureValue(Me,"Id") = Id
                    Set r = binder.ItemForWrite(Id, "PERSON")
                    If Not r Is Nothing Then
                        binder.WriteTo r
                        Id = binder.WriteItem(r,"PERSON")
                        IndexEntity Id
                    End If
                    If Database.CompleteTransaction Then
                        FillNomenclature
                        cRights.Id = Id
                        Selected = True
                        Mode = UCPerson_Mode_Slim
                        Save = True
                    End If
                On Error Goto 0
                End If
            Else
                Database.InvalidateTransaction ' This control has its own buttons, Just in case
            End If
        End Function
        Public Function Delete
            Delete = False
            If Id <> 0 Then
                If Database.BeginTransaction Then
                On Error Resume Next
                    Database.FailureValue(Me,"Id") = Id
                    binder.DeleteItem Id, "PERSON"
                    Database.DB.VExecute "DELETE FROM OBJECT_KEYWORD WHERE OBJECT_SID=$sid", 1, 0, id
                    Id = 0
                    If Database.CompleteTransaction Then
                        Mode = UCPerson_Mode_Slim
                        cRights.Id = Id
                        Selected = True
                        Delete = True
                    End If
                On Error Goto 0
                End If
            End If
        End Function

        Public Function Complete(bSuccess, Failure, Success)
            Complete = True
            If Not bSuccess Then
                Id = ConvertTo(vbLong, Failure("Id"))
                cRights.Id = Id
            End If
        End Function
        
        Private Sub AutoFillFields
            Dim sNames, I, arr
            If txtName1.Value <> "" Or txtName2.Value <> "" Or txtName3.Value <> "" Or txtTitle.Value <> "" Then
                Exit Sub
            End If
            sNames = Trim(CachedPattern)
            If sNames <> "" Then
                arr = Split(sNames," ")
                If UBound(arr) >= 0 Then
                    For I = 0 To UBound(arr)
                        If I < 3 Then
                            Select Case I
                                Case 0
                                    txtName1.Value = arr(I)
                                Case 1
                                    If UBound(arr) > 1 Then 
                                        txtName2.Value = arr(I)
                                    Else
                                        txtName3.Value = arr(I)
                                    End If
                                Case 2
                                    txtName3.Value = arr(I)
                            End Select
                        End If
                    Next
                End If
            End IF
        End Sub
        
        Sub ProcessPage
            Dim sNames, arr, I
            If IsPostBack Then
                If btnOk.Clicked Then
                    If Save Then
                        Selected = True
                        Mode = UCPerson_Mode_Slim
                    End If
                ElseIf btnCancel.Clicked Then
                    Mode = UCPerson_Mode_Slim
                    Selected = True
                ElseIf btnDelete.Clicked Then
                    Delete
                ElseIf btnClose.Clicked Then
                    If Mode = UCPerson_Mode_List Then CachedPattern = cPersons.NameFilter
                    Mode = UCPerson_Mode_Slim
                ElseIf btnOpen.Clicked Then
                    cPersons.Id = Id
                    Mode = UCPerson_Mode_List
                ElseIf btnAddNew.Clicked Then
                    Load 0
                    CachedPattern = cPersons.NameFilter
                    AutoFillFields
                ElseIf btnEdit.Clicked Then
                    Load Id
                ElseIf cPersons.Selected Then
                    Id = cPersons.Id
                    CachedPattern = cPersons.NameFilter
                    Mode = UCPerson_Mode_Slim
                    OvertakePartialRender Me
                    Selected = True
                End If
            End If
        End Sub
        
        ' POST VARIABLES
        Public Property Get Id
            Id = ConvertTo(vbLong,PostVariables.GetCtlVar(Me,"ID"))
        End Property
        Public Property Let Id(v)
            PostVariables.SetCtlVar Me, "ID", ConvertTo(vbLong,v)
            cRights.Id = v
        End Property
        Public Property Get Mode
            Mode = ConvertTo(vbLong,PostVariables.GetCtlVar(Me,"M"))
        End Property
        Public Property Let Mode(v)
            PostVariables.SetCtlVar Me, "M", ConvertTo(vbLong,v)
        End Property
        
        Public Property Get CachedPattern
            CachedPattern = ConvertTo(vbString,PostVariables.GetCtlVar(Me,"CP"))
        End Property
        Public Property Let CachedPattern(v)
            PostVariables.SetCtlVar Me, "CP", ConvertTo(vbString, v)
            cPersons.NameFilter = v
        End Property
        
        Public Property Get Value
            Value = Empty
            If Id <> 0 Then Value = Id
        End Property
        Public Property Let Value(v)
            Id = v
        End Property
        
        
        Public Property Get ClassType
            ClassType = "UCPerson" 
        End Property
        
        Public Sub Render
            %>
            <div <%= StyleAndCssString(CssClass,Style) %> id="<%= ClientId %>">
                <% RenderPartial %>
            </div>
            <%
        End Sub
        Public Sub RenderPartial
            If Hide Then Exit Sub
            RenderSlim
            If Mode = UCPerson_Mode_List Then ' List
                RenderList
            ElseIf Mode = UCPerson_Mode_Form Then ' Form
                RenderForm
            End If
        End Sub
        
        Private Sub RenderSlim
            Dim r, sPerson
            
            Set r = Database.DB.VExecute("SELECT * FROM PERSON WHERE SID=$SID AND LANGUAGE=$LANGUAGE AND " & SQLReadRights,1,1,Id,PageUILanguage)
            If r.Count > 0 Then
                sPerson = PersonName(r(1))
                Set rec = r(1)
            Else
                If Id <> 0 Then
                    sPerson = TR("(access denied)")
                Else
                    sPerson = TR("(not selected)")
                End If
                Set rec = CreateCollection
            End If
            %>
            <table class="<%= IfThenElse(Mode,"SelectorBarActive","SelectorBar") %>" style="width:100%;">
                <tr>
                    <td class="leftSide"><img class="pixel" src="<%= SysImage("pixel") %>" alt=""/></td>
                    <% If Not ReadOnly Then %>
                    <td nowrap><% btnAddNew.Render %></td>
                        <% 
                        If Id <> 0 Then 
                            %><td nowrap><% btnEdit.Render %></td><%
                        End If
                        %>
                    <% End If %>
                    <td nowrap width="100%"><%= HTMLEncode(Id & ": " & sPerson) %></td>
                    <% If Mode Then %>
                        <td nowrap><% btnClose.Render %></td>
                    <% Else %>
                        <td nowrap><% btnOpen.Render %></td>
                    <% End If %>
                    <td class="rightSide"><img class="pixel" src="<%= SysImage("pixel") %>" alt=""/></td>
                </tr>
            </table>
            <%
        End Sub
        
        Private Sub RenderList
            cPersons.Render
        End Sub
        
        Private Sub RenderForm
            %>
            <table class="form">
                <tr>
                    <th class="formCaption" colspan="2"><%= TR("Person data") %><% vldNamesPerson.Render %></th>
                </tr>
                <tr>
                    <th nowrap class="formLabel" colspan="2"><% cRights.Render %></th>
                </tr>
                <tr>
                    <th nowrap class="formLabel"><%= TR("Title") %></th>
                    <td nowrap class="formField"><% txtTitle.Render : vldTitle.Render %></td>
                </tr>
                <tr>
                    <th nowrap class="formLabel"><%= TR("First name") %></th>
                    <td nowrap class="formField"><% txtName1.Render : vldName1.Render %></td>
                </tr>
                <tr>
                    <th nowrap class="formLabel"><%= TR("Middle name") %></th>
                    <td nowrap class="formField"><% txtName2.Render : vldName2.Render %></td>
                </tr>
                <tr>
                    <th nowrap class="formLabel"><%= TR("Last name") %></th>
                    <td nowrap class="formField"><% txtName3.Render : vldName3.Render %></td>
                </tr>
                <tr>
                    <th nowrap class="formLabel" colspan="1"><%= TR("Comment") %></th>
                    <td nowrap class="formField" colspan="1"><% txtComment.Render : vldComment.Render %></td>
                </tr>
                <tr>
                    <th nowrap class="formLabel" colspan="1"><%= TR("Photo/Avatar") %></th>
                    <td nowrap class="formField" colspan="1"><% cImage.Render %></td>
                </tr>
                <tr>
                    <td nowrap class="formButtons" colspan="2">
                        <% 
                            btnOk.Render
                            btnCancel.Render
                            Response.Write "&nbsp;"
                            If Id <> 0 Then btnDelete.Render
                        %>
                    </td>
                </tr>
            </table>
            <%
        End Sub
    
    End Class

%>